# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2012-2016, by Tony Arcieri.
# Copyright, 2012, by Ryan LeCompte.
# Copyright, 2012, by Nicholas Evans.
# Copyright, 2012, by Dimitrij Denissenko.
# Copyright, 2013, by Chuck Remes.
# Copyright, 2013, by Ron Evans.
# Copyright, 2013, by Sean Gregory.
# Copyright, 2013, by Utenmiki.
# Copyright, 2013, by Jeremy Hinegardner.
# Copyright, 2014, by Larry Lv.
# Copyright, 2014, by Bruno Enten.
# Copyright, 2014-2022, by Samuel Williams.
# Copyright, 2014, by Mike Bourgeous.

require_relative "timers/version"

require_relative "timers/group"
require_relative "timers/wait"
